#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

Display *disp;
Window   root;
Colormap dcm;
GC       gc;
XColor   colour[256];
Pixmap   pixmap;

unsigned int rgb[256][3];

unsigned long plane_masks, pixels[256];

int draw_circle();
int draw_box();

char *dispname = NULL;
int   waitusec = 20000;
int   size     = 64;
int   colours  = 30;
int   (*draw_mode)() = draw_circle;

main( argc, argv )
int argc;
char **argv;
{
    int i,j,k;
    int col;
    int power;

    analyse_opts( argc, argv );
 
    if((disp = XOpenDisplay( dispname )) == NULL ){
	fprintf(stderr, "Can't open display.\n");
	exit(1);
    }

    root = RootWindow( disp, 0 );
    dcm  = DefaultColormap( disp, 0 );
    gc   = DefaultGC( disp, 0 );

    pixmap = draw_mode();
    pixmapOnRoot( disp, 0, pixmap );

    i = 0;
    power = 0;

    while(1){
	int k;

	i--;
	if( i<0 ) i=colours-1;
	power --;
	if( power<0 ){
	    power = colours;
	    col = random()%7+1;
	} else if ( (random()%(power+3))==0 ){
	    power = colours;
	}

	for(j=0, k=1; j<3; j++, k*=2){
	    if( col & k )
		rgb[i][j] = 65535/colours*power;
	    else 
		rgb[i][j] = 0;
	}
	
	for(j=0; j<colours; j++){
	    colour[j].pixel = pixels[j];
	    colour[j].red   = rgb[(j+i)%colours][0];
	    colour[j].blue  = rgb[(j+i)%colours][1];
	    colour[j].green = rgb[(j+i)%colours][2];
	    colour[j].flags = DoRed | DoGreen | DoBlue;
	}
	XStoreColors( disp, dcm, colour, colours );
	XFlush( disp );
	usleep(waitusec);
    }
}

draw_circle()
{
    int j;
    int   naname;
    Pixmap pixmap;

    pixmap = XCreatePixmap( disp, root, size, size,
			   DefaultDepth(disp, 0));
    
    naname = (int)(sqrt((double)(size*size)/2.0))+2;
    if( colours > naname ) colours = naname;

    XAllocColorCells( disp, dcm, False,
		     &plane_masks, 0, pixels, colours );

    for(j=naname; j>=0; j--){
	XSetForeground( disp, gc, pixels[j*colours/naname] );
	XFillArc( disp, pixmap, gc, size/2-j, size/2-j, 
		 j*2, j*2, 0, 360*64 );
    }
    return pixmap;
}

draw_box()
{
    int j;
    Pixmap pixmap;

    pixmap = XCreatePixmap( disp, root, size, size,
			   DefaultDepth(disp, 0));
    
    XAllocColorCells( disp, dcm, False,
		     &plane_masks, 0, pixels, colours );

    for(j=size/2+1; j>=0; j--){
	XSetForeground( disp, gc, pixels[j*colours/size] );
	XFillRectangle( disp, pixmap, gc, size/2-j, size/2-j, 
		 j*2, j*2);
    }
    return pixmap;
}

analyse_opts( argc, argv )
int argc;
char **argv;
{
    enum { OPT_DISPLAY, OPT_WIDTH, OPT_WAIT, OPT_COLOURS, OPT_MODE };

    static char *ident[10] = {	"-display",
	                        "-width",
            	                "-wait",
	                        "-colours",
				"-mode",    
	                        NULL };
    int i;
    int j;
    int found;

    j = 1;
    while( j < argc ){
	i = 0;
	found = 0;
	do{
	    if( ident[i] == NULL ){
		fprintf( stderr, "Opt error.\n");
		exit(1);
	    }
	    if( !strcmp( ident[i], argv[j]) ){
		found = -1;
		switch( i ){
		 case OPT_DISPLAY:
		    if( ++j < argc )
			dispname = argv[j];
		    else {
			fprintf( stderr, "Specify display name.\n");
			exit(1);
		    }
		    break;
		 case OPT_WIDTH:
		    if( ++j < argc )
			size = atoi( argv[j] );
		    else {
			fprintf( stderr, "Specify width.\n");
			exit(1);
		    }
		    break;
		 case OPT_WAIT:
		    if( ++j < argc )
			waitusec = atoi( argv[j] );
		    else {
			fprintf( stderr, "Specify wait.\n");
			exit(1);
		    }
		    break;
		 case OPT_COLOURS:
		    if( ++j < argc )
			colours = atoi( argv[j] );
		    else {
			fprintf( stderr, "Specify colours.\n");
			exit(1);
		    }
		    break;
                 case OPT_MODE:
		    if( ++j < argc ){
			if(!strcmp("circle", argv[j]))
			    draw_mode = draw_circle;
			else if (!strcmp("box", argv[j]))
			    draw_mode = draw_box;
			else {
			    fprintf( stderr, "Illegal mode.\n");
			    exit(1);
			}
		    } else {
			fprintf( stderr, "Specify mode.\n");
			exit(1);
		    }
		    break;
		 default:
		    fprintf(stderr, "Sorry, %s is not supported.\n", argv[j]);
		    exit(1);
		}
	    } else i++;
	} while ( !found );
	j++;
    }
}

